#include "QSimulationBPImage.h"
#include <QFileDialog>
#include <QMessageBox>
#include "TBPImageAlgCls.h"
#include "EchoDataFormat.h"
#include <boost/thread.hpp>
#include <thread>

QSimulationBPImage::QSimulationBPImage(QWidget *parent)
	: QDialog(parent)
{
	ui.setupUi(this);

	QObject::connect(ui.pushButtonEchoSelect, SIGNAL(clicked()), this, SLOT(onpushButtonEchoSelectClicked()));
	QObject::connect(ui.pushButtonImageSelect, SIGNAL(clicked()), this, SLOT(onpushButtonImageSelectClicked()));
	QObject::connect(ui.buttonBox, SIGNAL(accepted()), this, SLOT(onbtnaccepted()));
	QObject::connect(ui.buttonBox, SIGNAL(rejected()), this, SLOT(onbtnrejected()));

}

QSimulationBPImage::~QSimulationBPImage()
{}

void QSimulationBPImage::onpushButtonEchoSelectClicked()
{
	QString  fileNames = QFileDialog::getOpenFileName(
		this,                           // 
		tr(u8"ѡӰļ"),               // 
		QString(),                      // Ĭ·
		tr(u8"xml Files (*.xml);;All Files (*)")  // ļ
	);

	// ûѡļ
	if (!fileNames.isEmpty()) {
		QString message = "ѡļУ\n";
		this->ui.lineEditEchoPath->setText(fileNames);
	}
	else {
		QMessageBox::information(this, tr(u8"ûѡļ"), tr(u8"ûѡκļ"));
	}
}

void QSimulationBPImage::onpushButtonImageSelectClicked()
{
	QString  fileNames = QFileDialog::getSaveFileName(
		this,                           // 
		tr(u8"ѡӰļ"),               // 
		QString(),                      // Ĭ·
		tr(u8"All Files(*)")  // ļ
	);

	// ûѡļ
	if (!fileNames.isEmpty()) {
		this->ui.lineEditImagePath->setText(fileNames);
	}
	else {
		QMessageBox::information(this, tr(u8"ûѡļ"), tr(u8"ûѡκļ"));
	}
}

void QSimulationBPImage::onbtnaccepted()
{
	QString echofile = this->ui.lineEditEchoPath->text().trimmed();
	QString outImageFolder = getParantFromPath(this->ui.lineEditImagePath->text().trimmed());
	QString imagename = getFileNameFromPath(this->ui.lineEditImagePath->text().trimmed());
	std::shared_ptr<EchoL0Dataset> echoL0ds(new EchoL0Dataset);
	echoL0ds->Open(echofile);

	std::shared_ptr< SARSimulationImageL1Dataset> imagL1(new SARSimulationImageL1Dataset);
	imagL1->setCenterAngle(echoL0ds->getCenterAngle());
	imagL1->setCenterFreq(echoL0ds->getCenterFreq());
	imagL1->setNearRange(echoL0ds->getNearRange());
	imagL1->setRefRange((echoL0ds->getNearRange() + echoL0ds->getFarRange()) / 2);
	imagL1->setFarRange(echoL0ds->getFarRange());
	imagL1->setFs(echoL0ds->getFs());
	imagL1->setLookSide(echoL0ds->getLookSide());
	imagL1->OpenOrNew(outImageFolder, imagename, echoL0ds->getPluseCount(), echoL0ds->getPlusePoints());


	TBPImageAlgCls TBPimag;
	TBPimag.setEchoL0(echoL0ds);
	TBPimag.setImageL1(imagL1);
	long cpucore_num = std::thread::hardware_concurrency();
	TBPimag.setGPU(true);
	TBPimag.Process(cpucore_num);
	QMessageBox::information(this,u8"",u8"");
	
}

void QSimulationBPImage::onbtnrejected()
{
	this->close();
}
